If you have questions to which you cannot find the answer, you can use the mattermost channel to raise these questions to a larger audience.
You can find the contact information on the first page of this website.


## Training questions

???+ question "What should I do in case my training does not seem to converge or I see poor performance?"
     There can be several reasons why your model is not performing as good as you'd like it to perform. Firstly you should remember that you
     are probably working on a complicated problem for which you would not expect perfect performance in the first place. Hence some imperfections
     are always expected. Nevertheless here are some ideas on what you can do to improve your model:
     * Think about the problem from a different point of view. Sometimes rethinking what your model should actually do helps already a lot.
     * Think about the variables you provide to your model. Are these enough to learn about the underlying physics? Can you maybe engineer other useful variables?
     * Study the control plots. Don't focus on one particular metric. Look at the bigger picture. The distributions in the control plots can give you indications where the problems are.
     * Try hyperparameter optimisation.

???+ question "I observe spiky loss curves. What does this mean?"
     Spiky loss curves usually mean, that your model is very sensitive to small changes. Try to increase the batch size or decrease the learning rate to become less sensitive to these fluctuations.

???+ question "How is the splitting into several folds done?"
    The splitting into several folds is done using your config file's FoldSplitVariable' parameter. By default, this is set to `eventNumber`.
    The code then applies the `%` operator to `FoldSplitVariable` such that
    ```
    fold = FoldSplitVariable % Folds
    ```
    where `Folds` is also specified in your config file.
    This is done for **all** samples, including data.

???+ question "How can I train on a subset of samples and ignore others in the training of a classifier?"
    You can set the `TrainingLabel` parameter of samples you want to ignore in training to any negative value.
    They will subsequently have no direct impact on the training and will only be "piped through" to all control plots.

???+ question "How are negative weights handled in the code?"
     There are currently three ways to handle negative event weights. They are all steered via the `TreatNegWeights` parameter in the `GENERAL` block.
     * `None`: Negative weights will impact the training. From a physics point of view this is the recommended treatment. If negative loss values are observed, it is necessary to switch to another option.
     * `Scale`: Per sample all positive weights are scaled down to account for the effect of the negative weights on the sample. Afterwards all negative weights are set to zero. This is recommended if `None` does not work.
     * `SetZero`: All negative event weights are set to zero and hence have no impact on the training

## Injection questions

???+ question "How can I inject predictions back into my ntuples?"
     For the injection procedure check out [this manual](../Short_Walkthrough/RunInjection.md).

## Questions related to working on lxplus

???+ question "I run the code on lxplus, and after a while I get permission denied errors on `afs` or `eos`. What is happening?"
    This problem most likely comes down to a kerberos authentification problem. I.e. after a while your authentification
    token will become invalid, and you will lose your write permissions.
    This [twiki](https://twiki.cern.ch/twiki/bin/view/Main/Kerberos) can be helpful in solving your problem.
