# Virtual environment on Lxplus

**At the moment setting up virtual environments with all package dependencies and access to pyROOT can be quite difficult on lxplus**. Hence this is **not** the recommended way of settings things up on lxplus.

The following documentation displays one way of setting up a virtual environment but does not guarantee that things are working properly. For a more reliable setup have a look at the setup using LCG stacks.

## Creating the virtual environment

This you technically only need to do once. The following lines will create the virtual environment for you:

```sh
setupATLAS
lsetup "ROOT 6.22.00-python3-x86_64-centos7-gcc8-opt"
python3 -m venv py3_venv
```
Here we chose the name `py3_venv`. Obviously you can also give your environment another name. If you do, make sure you use that name in the following steps instead of `py3_venv`. 

## Installing packages within the virtual environment

Source your virtual environment via:

```sh
source py3_venv/bin/activate
```

Your command line should have changed now and you should see `(py3_venv)` at its start.
Now we continue with the installation of packages. Do

```sh
pip3 install --upgrade pip
```
This will provide you with an up-to-date version of pip.
Continue with the following command:

```sh
pip3 install tensorflow pandas uproot scikit-learn tables matplotlib pydot
```
This command will install the necessary packages that should work with the python intepreter you have previously set up.

Check if your installation was successful and if all needed packages are indeed there by sourcing the mva-trainer's setup script:

```sh
source setup.sh
```

If the scripts terminates without errors you made it. The installation was successful.

The `mva-trainer` scripts themselves can be used (e.g. the `Converter` from the `mva-trainer` directory) via:

```sh
python3 scripts/Converter.py -c config/<your_config_file>
```

If you have problem with python modules that are loaded from cvmfs instead from your virtual environement than please add this line to the setup script:

```
export PYTHONPATH=/full/path/to/venv-directory/py3_venv/lib/python3.7/site-packages:$PYTHONPATH
```

You can leave the virtual environment by typing:

```
deactivate
```

For future uses you now only need to source the virtual environment via the source command:

```sh
source py3_venv/bin/activate
```

