"""
This module handles binning optimisation as described
in https://cds.cern.ch/record/2296985/files/CERN-THESIS-2017-258.pdf
"""
from array import array

class TransfoBinning(object):
    """
    Class for binning optimisation
    """
    def __init__(self, hist_b, hist_s, zb, zs):
        self.m_hist_b = hist_b
        self.m_hist_s = hist_s
        self.m_zb = zb
        self.m_zs = zs
        self.m_Nb = self.m_hist_b.Integral()
        self.m_Ns = self.m_hist_s.Integral()

    def TransfoD(self):
        """
        Function to get the actual TransfoD binning

        returns: float array with bin edges
        """
        nb = 0
        ns = 0
        Binning = []
        half_binwidth = (self.m_hist_b.GetBinCenter(1)-self.m_hist_b.GetBinCenter(0))/2
        Binning.append(self.m_hist_b.GetBinCenter(self.m_hist_b.GetNbinsX()+1)-half_binwidth)
        for bin_id in reversed(range(1, self.m_hist_b.GetNbinsX()+1)):
            nb += self.m_hist_b.GetBinContent(bin_id)
            ns += self.m_hist_s.GetBinContent(bin_id)
            Z = self.m_zb*nb/self.m_Nb+self.m_zs*ns/self.m_Ns
            if Z > 1:
                Binning.append(self.m_hist_b.GetBinCenter(bin_id)-half_binwidth)
                nb = 0
                ns = 0
        Binning.append(self.m_hist_b.GetBinCenter(1)-half_binwidth)
        return array("d", Binning[::-1])
